# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

CONF = config.CONF


class PortForwardingTestJSON(base.BaseNetworkTest):

    required_extensions = ['router', 'floating-ip-port-forwarding']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

        # Create network, subnet, router and add interface
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.router = cls.create_router(data_utils.rand_name('router'),
                                       external_network_id=cls.ext_net_id)
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])

    @decorators.idempotent_id('829a446e-46bc-41ce-b442-6e428aeb3c19')
    def test_port_forwarding_life_cycle(self):
        fip = self.create_floatingip()
        port = self.create_port(self.network)
        # Create port forwarding for one TCP port
        created_pf = self.create_port_forwarding(
            fip['id'],
            internal_port_id=port['id'],
            internal_ip_address=port['fixed_ips'][0]['ip_address'],
            internal_port=1111, external_port=2222, protocol="tcp")
        self.assertEqual(1111, created_pf['internal_port'])
        self.assertEqual(2222, created_pf['external_port'])
        self.assertEqual('tcp', created_pf['protocol'])
        self.assertEqual(port['fixed_ips'][0]['ip_address'],
                         created_pf['internal_ip_address'])

        # Show created port forwarding
        body = self.client.get_port_forwarding(
            fip['id'], created_pf['id'])
        pf = body['port_forwarding']
        self.assertEqual(1111, pf['internal_port'])
        self.assertEqual(2222, pf['external_port'])
        self.assertEqual('tcp', pf['protocol'])
        self.assertEqual(port['fixed_ips'][0]['ip_address'],
                         pf['internal_ip_address'])

        # Update port forwarding
        body = self.client.update_port_forwarding(
            fip['id'], pf['id'], internal_port=3333)
        pf = body['port_forwarding']
        self.assertEqual(3333, pf['internal_port'])
        self.assertEqual(2222, pf['external_port'])
        self.assertEqual('tcp', pf['protocol'])
        self.assertEqual(port['fixed_ips'][0]['ip_address'],
                         pf['internal_ip_address'])

        # Now lets try to remove Floating IP with existing port forwarding,
        # this should fails
        self.assertRaises(exceptions.Conflict,
                          self.delete_floatingip, fip)

        # Delete port forwarding
        self.client.delete_port_forwarding(fip['id'], pf['id'])
        self.assertRaises(exceptions.NotFound,
                          self.client.get_port_forwarding,
                          fip['id'], pf['id'])

        # Now Floating IP should be deleted properly
        self.delete_floatingip(fip)
        self.assertRaises(exceptions.NotFound,
                          self.client.get_floatingip, fip['id'])

    @decorators.idempotent_id('aa842070-39ef-4b09-9df9-e723934f96f8')
    @utils.requires_ext(extension="expose-port-forwarding-in-fip",
                        service="network")
    def test_port_forwarding_info_in_fip_details(self):
        fip = self.create_floatingip()
        port = self.create_port(self.network)

        # Ensure that FIP don't have information about any port forwarding yet
        fip = self.client.show_floatingip(fip['id'])['floatingip']
        self.assertEqual(0, len(fip['port_forwardings']))

        # Now create port forwarding and ensure that it is visible in FIP's
        # details
        pf = self.create_port_forwarding(
            fip['id'],
            internal_port_id=port['id'],
            internal_ip_address=port['fixed_ips'][0]['ip_address'],
            internal_port=1111, external_port=2222, protocol="tcp")
        fip = self.client.show_floatingip(fip['id'])['floatingip']
        self.assertEqual(1, len(fip['port_forwardings']))
        self.assertEqual(1111, fip['port_forwardings'][0]['internal_port'])
        self.assertEqual(2222, fip['port_forwardings'][0]['external_port'])
        self.assertEqual('tcp', fip['port_forwardings'][0]['protocol'])
        self.assertEqual(port['fixed_ips'][0]['ip_address'],
                         fip['port_forwardings'][0]['internal_ip_address'])

        # Delete port forwarding and ensure that it's not in FIP's details
        # anymore
        self.client.delete_port_forwarding(fip['id'], pf['id'])
        fip = self.client.show_floatingip(fip['id'])['floatingip']
        self.assertEqual(0, len(fip['port_forwardings']))

    def _verify_created_pfs(self, fip_id, created_pfs):
        # Check that all PFs are visible in Floating IP details
        fip = self.client.show_floatingip(fip_id)['floatingip']
        self.assertEqual(len(created_pfs), len(fip['port_forwardings']))
        detail_enabled = utils.is_extension_enabled(
            'floating-ip-port-forwarding-detail', service='network')
        for pf in created_pfs:
            expected_pf = {
                'external_port': pf['external_port'],
                'internal_port': pf['internal_port'],
                'protocol': pf['protocol'],
                'internal_ip_address': pf['internal_ip_address']}
            if detail_enabled:
                expected_pf['id'] = pf['id']
                expected_pf['internal_port_id'] = pf['internal_port_id']
            self.assertIn(expected_pf, fip['port_forwardings'])

        # Test list of port forwardings
        port_forwardings = self.client.list_port_forwardings(
            fip['id'])['port_forwardings']
        self.assertEqual(len(created_pfs), len(port_forwardings))
        for pf in created_pfs:
            expected_pf = pf.copy()
            expected_pf.pop('client')
            expected_pf.pop('floatingip_id')
            self.assertIn(expected_pf, port_forwardings)

    def _create_and_validate_pf(self, fip_id, internal_port_id,
                                internal_ip_address, internal_port,
                                external_port, protocol):
        created_pf = self.create_port_forwarding(
            fip_id,
            internal_port_id=internal_port_id,
            internal_ip_address=internal_ip_address,
            internal_port=internal_port,
            external_port=external_port,
            protocol=protocol)
        self.assertEqual(internal_port, created_pf['internal_port'])
        self.assertEqual(external_port, created_pf['external_port'])
        self.assertEqual(protocol, created_pf['protocol'])
        self.assertEqual(internal_ip_address,
                         created_pf['internal_ip_address'])
        return created_pf

    @decorators.idempotent_id('8202cded-7e82-4420-9585-c091105404f6')
    def test_associate_2_port_forwardings_to_floating_ip(self):
        fip = self.create_floatingip()
        forwardings_data = [(1111, 2222), (3333, 4444)]
        created_pfs = []
        for data in forwardings_data:
            internal_port = data[0]
            external_port = data[1]
            port = self.create_port(self.network)
            created_pf = self._create_and_validate_pf(
                fip_id=fip['id'],
                internal_port_id=port['id'],
                internal_ip_address=port['fixed_ips'][0]['ip_address'],
                internal_port=internal_port, external_port=external_port,
                protocol="tcp")
            created_pfs.append(created_pf)
        self._verify_created_pfs(fip['id'], created_pfs)

    @decorators.idempotent_id('a7e6cc48-8a9b-49be-82fb-cef6f5c29381')
    def test_associate_port_forwarding_to_2_fixed_ips(self):
        fip = self.create_floatingip()
        port = self.create_port(self.network)
        internal_subnet_id = port['fixed_ips'][0]['subnet_id']
        # Add a second fixed_ip address to port (same subnet)
        port['fixed_ips'].append({'subnet_id': internal_subnet_id})
        port = self.update_port(port, fixed_ips=port['fixed_ips'])
        internal_ip_address1 = port['fixed_ips'][0]['ip_address']
        internal_ip_address2 = port['fixed_ips'][1]['ip_address']
        forwardings_data = [(4001, internal_ip_address1),
                            (4002, internal_ip_address2)]
        created_pfs = []
        for data in forwardings_data:
            external_port = data[0]
            internal_ip_address = data[1]
            created_pf = self._create_and_validate_pf(
                fip_id=fip['id'],
                internal_port_id=port['id'],
                internal_ip_address=internal_ip_address,
                internal_port=123, external_port=external_port,
                protocol="tcp")
            created_pfs.append(created_pf)
        self._verify_created_pfs(fip['id'], created_pfs)

    @decorators.idempotent_id('6a34e811-66d1-4f63-aa4d-9013f15deb62')
    def test_associate_port_forwarding_to_used_floating_ip(self):
        port_for_fip = self.create_port(self.network)
        fip = self.create_floatingip(port=port_for_fip)
        port = self.create_port(self.network)
        self.assertRaises(
            exceptions.Conflict,
            self.create_port_forwarding,
            fip['id'],
            internal_port_id=port['id'],
            internal_ip_address=port['fixed_ips'][0]['ip_address'],
            internal_port=1111, external_port=2222,
            protocol="tcp")

    @decorators.idempotent_id('4ca72d40-93e4-485f-a876-76caf33c1fe6')
    def test_associate_port_forwarding_to_port_with_fip(self):
        port = self.create_port(self.network)
        self.create_floatingip(port=port)
        fip_for_pf = self.create_floatingip()
        self.assertRaises(
            exceptions.Conflict,
            self.create_port_forwarding,
            fip_for_pf['id'],
            internal_port_id=port['id'],
            internal_ip_address=port['fixed_ips'][0]['ip_address'],
            internal_port=1111, external_port=2222,
            protocol="tcp")



# Copyright 2016 FUJITSU LIMITED
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import metering as metering_apidef
from neutron_lib.db import constants as db_const
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base

LONG_NAME_NG = 'x' * (db_const.NAME_FIELD_SIZE + 1)


class MeteringNegativeTestJSON(base.BaseAdminNetworkTest):

    required_extensions = [metering_apidef.ALIAS]

    @decorators.attr(type='negative')
    @decorators.idempotent_id('8b3f7c84-9d37-4771-8681-bfd2c07f3c2d')
    def test_create_metering_label_with_too_long_name(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.create_metering_label,
                          name=LONG_NAME_NG)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin import config

CONF = config.CONF


class FloatingIPNegativeTestJSON(base.BaseNetworkTest):

    required_extensions = ['router']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.ext_net_id = CONF.network.public_network_id
        # Create a network with a subnet connected to a router.
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)
        cls.router = cls.create_router(data_utils.rand_name('router'))
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.port = cls.create_port(cls.network)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0b5b8797-6de7-4191-905c-a48b888eb429')
    def test_associate_floatingip_with_port_with_floatingip(self):
        net = self.create_network()
        subnet = self.create_subnet(net)
        r = self.create_router('test')
        self.create_router_interface(r['id'], subnet['id'])
        self.client.update_router(
            r['id'],
            external_gateway_info={
                'network_id': self.ext_net_id})
        self.addCleanup(self.client.update_router, self.router['id'],
                        external_gateway_info={})
        port = self.create_port(net)
        floating_ip1 = self.create_floatingip()
        floating_ip2 = self.create_floatingip()
        self.client.update_floatingip(floating_ip1['id'],
                                      port_id=port['id'])
        self.assertRaises(lib_exc.Conflict, self.client.update_floatingip,
                          floating_ip2['id'], port_id=port['id'])



# Copyright 2011, VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# Borrowed from nova code base, more utilities will be added/borrowed as and
# when needed.

"""Utilities and helper functions."""

import threading
import time
try:
    import urlparse
except ImportError:
    from urllib import parse as urlparse

from neutron_lib._i18n import _
from oslo_log import log
from tempest.lib import exceptions

from neutron_tempest_plugin import config


SCHEMA_PORT_MAPPING = {
    "http": 80,
    "https": 443,
}
CONF = config.CONF
LOG = log.getLogger(__name__)


class classproperty:
    def __init__(self, f):
        self.func = f

    def __get__(self, obj, owner):
        return self.func(owner)


class WaitTimeout(Exception):
    """Default exception coming from wait_until_true() function."""


class LockWithTimer:
    def __init__(self, threshold):
        self._threshold = threshold
        self.timestamp = 0
        self._lock = threading.Lock()

    def acquire(self):
        return self._lock.acquire(False)

    def release(self):
        return self._lock.release()

    def time_to_wait(self):
        return self.timestamp - time.time() + self._threshold


def wait_until_true(predicate, timeout=60, sleep=1, exception=None):
    """Wait until callable predicate is evaluated as True

    :param predicate: Callable deciding whether waiting should continue.
       Best practice is to instantiate predicate with functools.partial()
    :param timeout: Timeout in seconds how long should function wait.
    :param sleep: Polling interval for results in seconds.
    :param exception: Exception instance to raise on timeout. If None is passed
                      (default) then WaitTimeout exception is raised.
    """
    start_time = time.time()
    while not predicate():
        elapsed_time = time.time() - start_time
        if elapsed_time > timeout:
            raise exception if exception else WaitTimeout(
                _("Timed out after %d seconds") % timeout
            )
        time.sleep(sleep)


def override_class(overriden_class, overrider_class):
    """Override class definition with a MixIn class

    If overriden_class is not a subclass of overrider_class then it creates
    a new class that has as bases overrider_class and overriden_class.
    """

    if not issubclass(overriden_class, overrider_class):
        name = overriden_class.__name__
        bases = (overrider_class, overriden_class)
        overriden_class = type(name, bases, {})
    return overriden_class


def normalize_url(url):
    """Normalize url without port with schema default port

    """
    parse_result = urlparse.urlparse(url)
    (scheme, netloc, url, params, query, fragment) = parse_result
    port = parse_result.port
    if scheme in SCHEMA_PORT_MAPPING and not port:
        netloc = netloc + ":" + str(SCHEMA_PORT_MAPPING[scheme])
    return urlparse.urlunparse((scheme, netloc, url, params, query, fragment))


def kill_nc_process(ssh_client):
    cmd = "killall -q nc"
    try:
        ssh_client.exec_command(cmd)
    except exceptions.SSHExecCommandFailed:
        pass


def process_is_running(ssh_client, process_name):
    try:
        ssh_client.exec_command("pidof %s" % process_name)
        return True
    except exceptions.SSHExecCommandFailed:
        return False


class StatefulConnection:
    """Class to test connection that should remain opened

    Can be used to perform some actions while the initiated connection
    remain opened
    """

    def __init__(self, client_ssh, server_ssh, target_ip, target_port):
        self.client_ssh = client_ssh
        self.server_ssh = server_ssh
        self.ip = target_ip
        self.port = target_port
        self.connection_started = False
        self.test_attempt = 0
        self.test_timeout = 10
        self.test_sleep = 1

    def __enter__(self):
        return self

    @property
    def test_str(self):
        return 'attempt_{}'.format(str(self.test_attempt).zfill(3))

    def _start_connection(self):
        if CONF.neutron_plugin_options.default_image_is_advanced:
            server_exec_method = self.server_ssh.execute_script
            client_exec_method = self.client_ssh.execute_script
        else:
            server_exec_method = self.server_ssh.exec_command
            client_exec_method = self.client_ssh.exec_command

        self.server_ssh.exec_command(
                'echo "{}" > input.txt'.format(self.test_str))
        server_exec_method('tail -f input.txt | sudo nc -lp '
                '{} &> output.txt & sleep 1'.format(self.port))
        self.client_ssh.exec_command(
                'echo "{}" > input.txt'.format(self.test_str))
        client_exec_method('tail -f input.txt | sudo nc {} {} &>'
                'output.txt &'.format(self.ip, self.port))

    def _nc_is_running(self):
        server = process_is_running(self.server_ssh, 'nc')
        client = process_is_running(self.client_ssh, 'nc')
        if client and server:
            return True
        else:
            return False

    def _test_connection(self):
        if not self.connection_started:
            self._start_connection()
        else:
            self.server_ssh.exec_command(
                    'echo "{}" >> input.txt'.format(self.test_str))
            self.client_ssh.exec_command(
                    'echo "{}" >> input.txt & sleep 1'.format(self.test_str))
        wait_until_true(self._nc_is_running,
                        timeout=self.test_timeout,
                        sleep=self.test_sleep)
        try:
            LOG.info("Checking connectivity between server and client -"
                    " attempt %d", self.test_attempt)
            self.server_ssh.exec_command(
                    'grep {} output.txt'.format(self.test_str))
            self.client_ssh.exec_command(
                    'grep {} output.txt'.format(self.test_str))
            if not self.should_pass:
                LOG.warning("attempt %d succeed while it should fail",
                            self.test_attempt)
                return False
            else:
                if not self.connection_started:
                    self.connection_started = True
                LOG.info("attempt %d succeed as it expected",
                         self.test_attempt)
                return True
        except exceptions.SSHExecCommandFailed:
            if self.should_pass:
                LOG.warning("attempt %d failed while it should pass",
                            self.test_attempt)
                return False
            else:
                LOG.info("attempt %d failed as it expected",
                         self.test_attempt)
                return True
        finally:
            self.test_attempt += 1

    def test_connection(self, should_pass=True, timeout=10, sleep_timer=1):
        self.should_pass = should_pass
        self.test_timeout = timeout
        self.test_sleep = sleep_timer
        wait_until_true(self._test_connection,
                        timeout=self.test_timeout,
                        sleep=self.test_sleep)

    def __exit__(self, type, value, traceback):
        self.server_ssh.exec_command('sudo killall nc || killall nc || '
                                     'echo "True"')
        self.server_ssh.exec_command(
                'sudo killall tail || killall tail || echo "True"')
        self.client_ssh.exec_command('sudo killall nc || killall nc || '
                                     'echo "True"')
        self.client_ssh.exec_command(
                'sudo killall tail || killall tail || echo "True"')



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from neutron_tempest_plugin.api import base


class SecGroupAdminTest(base.BaseNetworkTest):
    required_extensions = ['security-group']
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.admin_client = cls.os_admin.network_client
        cls.identity_admin_client = cls.os_admin.projects_client

    @decorators.idempotent_id('44f1e1c4-af10-4aa0-972f-87c1c8fa25cc')
    def test_security_group_recreated_on_port_update(self):
        network = self.create_network()
        self.create_subnet(network)
        port = self.create_port(network, security_groups=[])
        for sg in self.client.list_security_groups()['security_groups']:
            if sg['name'] == 'default':
                self.admin_client.delete_security_group(sg['id'])
        self.update_port(port, name='update')
        names = [
            sg['name']
            for sg in self.client.list_security_groups()['security_groups']
        ]
        self.assertIn('default', names)



# Copyright (c) 2017 x-ion GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import importlib
import ipaddress

from oslo_log import log
from tempest.common import utils
from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base as base_api
from neutron_tempest_plugin import config
from neutron_tempest_plugin.scenario import base
from neutron_tempest_plugin.scenario import constants


CONF = config.CONF
LOG = log.getLogger(__name__)


# Note(jh): Need to do a bit of juggling here in order to avoid failures
# when designate_tempest_plugin is not available
try:
    dns_base = importlib.import_module('designate_tempest_plugin.tests.base')
except ImportError:
    dns_base = None
try:
    dns_waiters = importlib.import_module(
        'designate_tempest_plugin.common.waiters')
except ImportError:
    dns_waiters = None
try:
    dns_data_utils = importlib.import_module(
        'designate_tempest_plugin.data_utils')
except ImportError:
    dns_data_utils = None


if dns_base:
    DNSMixin = dns_base.BaseDnsV2Test
else:
    DNSMixin = object


class BaseDNSIntegrationTests(base.BaseTempestTestCase, DNSMixin):
    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.zone_client = cls.os_tempest.dns_v2.ZonesClient()
        cls.recordset_client = cls.os_tempest.dns_v2.RecordsetClient()
        cls.query_client.build_timeout = 60

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not ('designate' in CONF.service_available and
                CONF.service_available.designate):
            raise cls.skipException("Designate support is required")
        if not (dns_base and dns_waiters):
            raise cls.skipException("Designate tempest plugin is missing")

    @classmethod
    @utils.requires_ext(extension="dns-integration", service="network")
    def resource_setup(cls):
        super().resource_setup()
        cls.zone_name = dns_data_utils.rand_zone_name(
            name="basednsintegrationtests")
        cls.zone = cls.zone_client.create_zone(
            name=cls.zone_name, wait_until='ACTIVE')[1]
        cls.addClassResourceCleanup(
            cls.zone_client.delete_zone, cls.zone['id'],
            ignore_errors=lib_exc.NotFound)
        cls.network = cls.create_network(dns_domain=cls.zone['name'])
        cls.subnet = cls.create_subnet(cls.network)
        cls.subnet_v6 = cls.create_subnet(cls.network, ip_version=6)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair()

    def _create_floatingip_with_dns(self, dns_name):
        return self.create_floatingip(client=self.os_primary.network_client,
                                      dns_name=dns_name,
                                      dns_domain=self.zone['name'])

    def _create_server(self, name=None):
        port = self.create_port(self.network)
        server = self.create_server(
            flavor_ref=CONF.compute.flavor_ref,
            image_ref=CONF.compute.image_ref,
            key_name=self.keypair['name'], name=name,
            networks=[{'port': port['id']}])['server']
        waiters.wait_for_server_status(self.os_primary.servers_client,
                                       server['id'],
                                       constants.SERVER_STATUS_ACTIVE)
        fip = self.create_floatingip(port=port)
        return {'port': port, 'fip': fip, 'server': server}

    def _check_type_in_recordsets(self, zone_id, rec_type):
        types = [rec['type'] for rec in self.recordset_client.list_recordset(
            zone_id)[1]['recordsets']]
        if rec_type in types:
            return True
        return False

    def _wait_for_type_in_recordsets(self, zone_id, type):
        test_utils.call_until_true(
            func=self._check_type_in_recordsets, zone_id=zone_id,
            rec_type=type, duration=self.query_client.build_timeout,
            sleep_for=5)

    def _check_recordset_deleted(
            self, recordset_client, zone_id, recordset_id):
        return test_utils.call_and_ignore_notfound_exc(
            recordset_client.show_recordset, zone_id, recordset_id) is None

    def _verify_designate_recordset(
            self, address, found=True, record_type='A'):
        if found:
            self._wait_for_type_in_recordsets(self.zone['id'], record_type)
            recordsets = self.recordset_client.list_recordset(
                self.zone['id'])[1]['recordsets']
            relevant_type = [rec for rec in recordsets if
                             rec['type'] == record_type]
            self.assertTrue(
                relevant_type,
                'Failed no {} type recordset has been detected in the '
                'Designate DNS DB'.format(record_type))
            rec_id = [rec['id'] for rec in relevant_type if address in
                      str(rec['records'])][0]
            self.assertTrue(
                rec_id, 'Record of type:{} with IP:{} was not detected in '
                        'the Designate DNS DB'.format(record_type, address))
            dns_waiters.wait_for_recordset_status(
                self.recordset_client, self.zone['id'], rec_id, 'ACTIVE')
        else:
            rec_id = None
            recordsets = self.recordset_client.list_recordset(
                self.zone['id'])[1]['recordsets']
            relevant_type = [rec for rec in recordsets if
                             rec['type'] == record_type]
            if relevant_type:
                rec_id = [rec['id'] for rec in relevant_type if
                          address in str(rec['records'])][0]
            if rec_id:
                recordset_exists = test_utils.call_until_true(
                    func=self._check_recordset_deleted,
                    recordset_client=self.recordset_client,
                    zone_id=self.zone['id'], recordset_id=rec_id,
                    duration=self.query_client.build_timeout, sleep_for=5)
                self.assertTrue(
                    recordset_exists,
                    'Failed, recordset type:{} and ID:{} is still exist in '
                    'the Designate DNS DB'.format(record_type, rec_id))

    def _verify_dns_records(self, address, name, found=True, record_type='A'):
        client = self.query_client
        forward = name + '.' + self.zone['name']
        reverse = ipaddress.ip_address(address).reverse_pointer
        record_types_to_check = [record_type, 'PTR']
        for rec_type in record_types_to_check:
            try:
                if rec_type == 'PTR':
                    dns_waiters.wait_for_query(
                        client, reverse, rec_type, found)
                else:
                    dns_waiters.wait_for_query(
                        client, forward, rec_type, found)
            except Exception as e:
                LOG.error(e)
                self._verify_designate_recordset(address, found, rec_type)
        if not found:
            return
        fwd_response = client.query(forward, record_type)
        rev_response = client.query(reverse, 'PTR')
        for r in fwd_response:
            for rr in r.answer:
                self.assertIn(address, rr.to_text())
        for r in rev_response:
            for rr in r.answer:
                self.assertIn(forward, rr.to_text())


class DNSIntegrationTests(BaseDNSIntegrationTests):
    @decorators.idempotent_id('850ee378-4b5a-4f71-960e-0e7b12e03a34')
    def test_server_with_fip(self):
        name = data_utils.rand_name('server-test')
        server = self._create_server(name=name)
        server_ip = server['fip']['floating_ip_address']
        self._verify_dns_records(server_ip, name)
        self.delete_floatingip(server['fip'])
        self._verify_dns_records(server_ip, name, found=False)

    @decorators.idempotent_id('a8f2fade-8d5c-40f9-80f0-3de4b8d91985')
    def test_fip(self):
        name = data_utils.rand_name('fip-test')
        fip = self._create_floatingip_with_dns(name)
        addr = fip['floating_ip_address']
        self._verify_dns_records(addr, name)
        self.delete_floatingip(fip)
        self._verify_dns_records(addr, name, found=False)


class DNSIntegrationAdminTests(BaseDNSIntegrationTests,
                               base_api.BaseAdminNetworkTest):

    credentials = ['primary', 'admin']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        segmentation_id = CONF.designate_feature_enabled.segmentation_id
        cls.network2 = cls.create_network(
            dns_domain=cls.zone['name'], provider_network_type='vxlan',
            provider_segmentation_id=segmentation_id)
        cls.subnet2 = cls.create_subnet(cls.network2)

    def _verify_dns_assignment(self, port):
        expected_fqdn = '%s.%s' % (port['dns_name'], self.zone['name'])
        self.assertEqual(expected_fqdn, port['dns_assignment'][0]['fqdn'])

    @decorators.idempotent_id('fa6477ce-a12b-41da-b671-5a3bbdafab07')
    def test_port_on_special_network(self):
        name = data_utils.rand_name('port-test')
        port = self.create_port(self.network2,
                                dns_name=name)
        self._verify_dns_assignment(port)
        addr = port['fixed_ips'][0]['ip_address']
        self._verify_dns_records(addr, name)
        self.client.delete_port(port['id'])
        self._verify_dns_records(addr, name, found=False)

    @decorators.idempotent_id('d44cd5b8-ac67-4965-96ff-cb77ab6aea8b')
    def test_fip_admin_delete(self):
        name = data_utils.rand_name('fip-test')
        fip = self._create_floatingip_with_dns(name)
        addr = fip['floating_ip_address']
        self._verify_dns_records(addr, name)
        self.delete_floatingip(fip, client=self.admin_client)
        self._verify_dns_records(addr, name, found=False)


class DNSIntegrationExtraTests(BaseDNSIntegrationTests):

    required_extensions = ["subnet-dns-publish-fixed-ip"]

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.network2 = cls.create_network(
            name=data_utils.rand_name('dns_integration_net'))
        cls.subnet2 = cls.create_subnet(cls.network2, cidr='10.123.151.0/24')
        cls.subnet2_v6 = cls.create_subnet(cls.network2,
                                           ip_version=6,
                                           dns_publish_fixed_ip=True)

    @decorators.idempotent_id('e10e0e5d-69ac-4172-b39f-27ab344b7f99')
    def test_port_with_publishing_subnet(self):
        name = data_utils.rand_name('port-test')
        port = self.create_port(self.network2,
                                dns_domain=self.zone['name'],
                                dns_name=name)
        fixed_ips = port['fixed_ips']
        if fixed_ips[1]['subnet_id'] == self.subnet2_v6['id']:
            v6_index = 1
        else:
            v6_index = 0
        addr_v4 = port['fixed_ips'][1 - v6_index]['ip_address']
        addr_v6 = port['fixed_ips'][v6_index]['ip_address']
        self._verify_dns_records(addr_v6, name, record_type='AAAA')
        self._verify_dns_records(addr_v4, name, found=False)
        self.client.delete_port(port['id'])
        self._verify_dns_records(addr_v6, name, record_type='AAAA',
                                 found=False)
        self.client.update_subnet(
            self.subnet2['id'], dns_publish_fixed_ip=True)
        port = self.create_port(self.network2,
                                dns_domain=self.zone['name'],
                                dns_name=name)
        addr_v4 = port['fixed_ips'][1 - v6_index]['ip_address']
        self._verify_dns_records(addr_v4, name, record_type='A')
        self.client.delete_port(port['id'])
        self._verify_dns_records(addr_v4, name, record_type='A',
                                 found=False)


class DNSIntegrationDomainPerProjectTests(BaseDNSIntegrationTests):

    credentials = ['primary', 'admin']

    required_extensions = ['subnet-dns-publish-fixed-ip',
                           'dns-integration-domain-keywords']

    @classmethod
    def resource_setup(cls):
        super(BaseDNSIntegrationTests, cls).resource_setup()
        cls.name = data_utils.rand_name('test-domain')
        cls.zone_name = "%s.%s.%s.zone." % (cls.client.user_id,
                                        cls.client.project_id,
                                        cls.name)
        dns_domain_template = "<user_id>.<project_id>.%s.zone." % cls.name
        cls.zone = cls.zone_client.create_zone(
            name=cls.zone_name, wait_until='ACTIVE')[1]
        cls.addClassResourceCleanup(cls.zone_client.delete_zone,
            cls.zone['id'], ignore_errors=lib_exc.NotFound)
        cls.network = cls.create_network(dns_domain=dns_domain_template)
        cls.subnet = cls.create_subnet(cls.network,
                                       dns_publish_fixed_ip=True)
        cls.subnet_v6 = cls.create_subnet(cls.network,
                                          ip_version=6,
                                          dns_publish_fixed_ip=True)
        cls.router = cls.create_router_by_client()
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        cls.keypair = cls.create_keypair()

    @decorators.idempotent_id('43a67509-3161-4125-8f2c-0d4a67599721')
    def test_port_with_dns_name(self):
        name = data_utils.rand_name('port-test')
        port = self.create_port(self.network,
                                dns_name=name)
        addr = port['fixed_ips'][0]['ip_address']
        self._verify_dns_records(addr, name)
        self.client.delete_port(port['id'])
        self._verify_dns_records(addr, name, found=False)

    @decorators.idempotent_id('ac89db9b-5ca4-43bd-85ba-40fbeb47e208')
    def test_fip_admin_delete(self):
        name = data_utils.rand_name('fip-test')
        fip = self._create_floatingip_with_dns(name)
        addr = fip['floating_ip_address']
        self._verify_dns_records(addr, name)
        self.delete_floatingip(fip, client=self.admin_client)
        self._verify_dns_records(addr, name, found=False)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest import config
from tempest.lib import decorators

from neutron_tempest_plugin.api import base


CONF = config.CONF


class ExtensionsTest(base.BaseNetworkTest):

    def _test_list_extensions_includes(self, exts):
        body = self.client.list_extensions()
        extensions = {ext_['alias'] for ext_ in body['extensions']}
        self.assertNotEmpty(extensions, "Extension list returned is empty")
        for ext in exts:
            ext_enabled = utils.is_extension_enabled(ext, "network")
            if ext_enabled:
                self.assertIn(ext, extensions)
            else:
                self.assertNotIn(ext, extensions)

    @decorators.idempotent_id('262420b7-a4bb-4a3e-b4b5-e73bad18df8c')
    def test_list_extensions_sorting(self):
        self._test_list_extensions_includes(['sorting'])

    @decorators.idempotent_id('19db409e-a23f-445d-8bc8-ca3d64c84706')
    def test_list_extensions_pagination(self):
        self._test_list_extensions_includes(['pagination'])

    @decorators.idempotent_id('155b7bc2-e358-4dd8-bf3e-1774c084567f')
    def test_list_extensions_project_id(self):
        self._test_list_extensions_includes(['project-id'])

    @decorators.idempotent_id('c7597fac-2404-45b1-beb4-523c8b1d4604')
    def test_list_extensions_includes_all(self):
        extensions = CONF.network_feature_enabled.api_extensions
        if not extensions:
            raise self.skipException("Extension list is empty")
        if extensions[0] == 'all':
            raise self.skipException("No lists of enabled extensions provided")

        self._test_list_extensions_includes(extensions)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base

SUBNETPOOL_NAME = 'smoke-subnetpool'
SUBNET_NAME = 'smoke-subnet'


class SubnetPoolsTestBase(base.BaseAdminNetworkTest):

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        min_prefixlen = '29'
        prefixes = ['10.11.12.0/24']
        cls._subnetpool_data = {'prefixes': prefixes,
                                'min_prefixlen': min_prefixlen}

    @classmethod
    def _create_subnetpool(cls, is_admin=False, **kwargs):
        if 'name' not in kwargs:
            name = data_utils.rand_name(SUBNETPOOL_NAME)
        else:
            name = kwargs.pop('name')

        if 'prefixes' not in kwargs:
            kwargs['prefixes'] = cls._subnetpool_data['prefixes']

        if 'min_prefixlen' not in kwargs:
            kwargs['min_prefixlen'] = cls._subnetpool_data['min_prefixlen']

        return cls.create_subnetpool(name=name, is_admin=is_admin, **kwargs)


class SubnetPoolsTest(SubnetPoolsTestBase):

    min_prefixlen = '28'
    max_prefixlen = '30'
    _ip_version = 4
    subnet_cidr = '10.11.12.0/29'
    new_prefix = '10.11.15.0/24'
    larger_prefix = '10.11.0.0/16'

    """Test Subnet Pools

    Tests the following operations in the Neutron API using the REST client for
    Neutron:

        create a subnetpool for a tenant
        list tenant's subnetpools
        show a tenant subnetpool details
        subnetpool update
        delete a subnetpool

        All subnetpool tests are run once with ipv4 and once with ipv6.

    v2.0 of the Neutron API is assumed.

    """

    def _new_subnetpool_attributes(self):
        new_name = data_utils.rand_name(SUBNETPOOL_NAME)
        return {'name': new_name, 'min_prefixlen': self.min_prefixlen,
                'max_prefixlen': self.max_prefixlen}

    def _check_equality_updated_subnetpool(self, expected_values,
                                           updated_pool):
        self.assertEqual(expected_values['name'],
                         updated_pool['name'])
        self.assertEqual(expected_values['min_prefixlen'],
                         updated_pool['min_prefixlen'])
        self.assertEqual(expected_values['max_prefixlen'],
                         updated_pool['max_prefixlen'])
        # expected_values may not contains all subnetpool values
        if 'prefixes' in expected_values:
            self.assertEqual(expected_values['prefixes'],
                             updated_pool['prefixes'])

    @decorators.idempotent_id('6e1781ec-b45b-4042-aebe-f485c022996e')
    def test_create_list_subnetpool(self):
        created_subnetpool = self._create_subnetpool()
        body = self.client.list_subnetpools()
        subnetpools = body['subnetpools']
        self.assertIn(created_subnetpool['id'],
                      [sp['id'] for sp in subnetpools],
                      "Created subnetpool id should be in the list")
        self.assertIn(created_subnetpool['name'],
                      [sp['name'] for sp in subnetpools],
                      "Created subnetpool name should be in the list")

    @decorators.idempotent_id('c72c1c0c-2193-4aca-ddd4-b1442640bbbb')
    @utils.requires_ext(extension="standard-attr-description",
                       service="network")
    def test_create_update_subnetpool_description(self):
        body = self._create_subnetpool(description='d1')
        self.assertEqual('d1', body['description'])
        sub_id = body['id']
        subnet_pools = [x for x in
            self.client.list_subnetpools()['subnetpools'] if x['id'] == sub_id]
        body = subnet_pools[0]
        self.assertEqual('d1', body['description'])
        body = self.client.update_subnetpool(sub_id, description='d2')
        self.assertEqual('d2', body['subnetpool']['description'])
        subnet_pools = [x for x in
            self.client.list_subnetpools()['subnetpools'] if x['id'] == sub_id]
        body = subnet_pools[0]
        self.assertEqual('d2', body['description'])

    @decorators.idempotent_id('741d08c2-1e3f-42be-99c7-0ea93c5b728c')
    def test_get_subnetpool(self):
        created_subnetpool = self._create_subnetpool()
        prefixlen = self._subnetpool_data['min_prefixlen']
        body = self.client.show_subnetpool(created_subnetpool['id'])
        subnetpool = body['subnetpool']
        self.assertEqual(created_subnetpool['name'], subnetpool['name'])
        self.assertEqual(created_subnetpool['id'], subnetpool['id'])
        self.assertEqual(prefixlen, subnetpool['min_prefixlen'])
        self.assertEqual(prefixlen, subnetpool['default_prefixlen'])
        self.assertFalse(subnetpool['shared'])

    @decorators.idempotent_id('5bf9f1e2-efc8-4195-acf3-d12b2bd68dd3')
    @utils.requires_ext(extension="project-id", service="network")
    def test_show_subnetpool_has_project_id(self):
        subnetpool = self._create_subnetpool()
        body = self.client.show_subnetpool(subnetpool['id'])
        show_subnetpool = body['subnetpool']
        self.assertIn('project_id', show_subnetpool)
        self.assertIn('tenant_id', show_subnetpool)
        self.assertEqual(self.client.project_id, show_subnetpool['project_id'])
        self.assertEqual(self.client.project_id, show_subnetpool['tenant_id'])

    @decorators.idempotent_id('764f1b93-1c4a-4513-9e7b-6c2fc5e9270c')
    def test_tenant_update_subnetpool(self):
        created_subnetpool = self._create_subnetpool()
        pool_id = created_subnetpool['id']
        subnetpool_data = self._new_subnetpool_attributes()
        self.client.update_subnetpool(created_subnetpool['id'],
                                      **subnetpool_data)

        body = self.client.show_subnetpool(pool_id)
        subnetpool = body['subnetpool']
        self._check_equality_updated_subnetpool(subnetpool_data,
                                                subnetpool)
        self.assertFalse(subnetpool['shared'])

    @decorators.idempotent_id('4b496082-c992-4319-90be-d4a7ce646290')
    def test_update_subnetpool_prefixes_append(self):
        # We can append new prefixes to subnetpool
        create_subnetpool = self._create_subnetpool()
        pool_id = create_subnetpool['id']
        old_prefixes = self._subnetpool_data['prefixes']
        new_prefixes = old_prefixes[:]
        new_prefixes.append(self.new_prefix)
        subnetpool_data = {'prefixes': new_prefixes}
        self.client.update_subnetpool(pool_id, **subnetpool_data)
        body = self.client.show_subnetpool(pool_id)
        prefixes = body['subnetpool']['prefixes']
        self.assertIn(self.new_prefix, prefixes)
        self.assertIn(old_prefixes[0], prefixes)

    @decorators.idempotent_id('2cae5d6a-9d32-42d8-8067-f13970ae13bb')
    def test_update_subnetpool_prefixes_extend(self):
        # We can extend current subnetpool prefixes
        created_subnetpool = self._create_subnetpool()
        pool_id = created_subnetpool['id']
        old_prefixes = self._subnetpool_data['prefixes']
        subnetpool_data = {'prefixes': [self.larger_prefix]}
        self.client.update_subnetpool(pool_id, **subnetpool_data)
        body = self.client.show_subnetpool(pool_id)
        prefixes = body['subnetpool']['prefixes']
        self.assertIn(self.larger_prefix, prefixes)
        self.assertNotIn(old_prefixes[0], prefixes)

    @decorators.idempotent_id('d70c6c35-913b-4f24-909f-14cd0d29b2d2')
    def test_admin_create_shared_subnetpool(self):
        created_subnetpool = self._create_subnetpool(is_admin=True,
                                                     shared=True)
        pool_id = created_subnetpool['id']
        # Shared subnetpool can be retrieved by tenant user.
        body = self.client.show_subnetpool(pool_id)
        subnetpool = body['subnetpool']
        self.assertEqual(created_subnetpool['name'], subnetpool['name'])
        self.assertTrue(subnetpool['shared'])

    def _create_subnet_from_pool(self, subnet_values=None, pool_values=None):
        if pool_values is None:
            pool_values = {}

        created_subnetpool = self._create_subnetpool(**pool_values)
        pool_id = created_subnetpool['id']
        subnet_name = data_utils.rand_name(SUBNETPOOL_NAME)
        network = self.create_network()
        subnet_kwargs = {'name': subnet_name,
                         'subnetpool_id': pool_id}
        if subnet_values:
            subnet_kwargs.update(subnet_values)
        # not creating the subnet using the base.create_subnet because
        # that function needs to be enhanced to support subnet_create when
        # prefixlen and subnetpool_id is specified.
        body = self.client.create_subnet(
            network_id=network['id'],
            ip_version=self._ip_version,
            **subnet_kwargs)
        subnet = body['subnet']
        return pool_id, subnet

    @decorators.idempotent_id('1362ed7d-3089-42eb-b3a5-d6cb8398ee77')
    def test_create_subnet_from_pool_with_prefixlen(self):
        subnet_values = {"prefixlen": self.max_prefixlen}
        pool_id, subnet = self._create_subnet_from_pool(
            subnet_values=subnet_values)
        cidr = str(subnet['cidr'])
        self.assertEqual(pool_id, subnet['subnetpool_id'])
        self.assertTrue(cidr.endswith(str(self.max_prefixlen)))

    @decorators.idempotent_id('86b86189-9789-4582-9c3b-7e2bfe5735ee')
    def test_create_subnet_from_pool_with_subnet_cidr(self):
        subnet_values = {"cidr": self.subnet_cidr}
        pool_id, subnet = self._create_subnet_from_pool(
            subnet_values=subnet_values)
        cidr = str(subnet['cidr'])
        self.assertEqual(pool_id, subnet['subnetpool_id'])
        self.assertEqual(cidr, self.subnet_cidr)

    @decorators.idempotent_id('83f76e3a-9c40-40c2-a015-b7c5242178d8')
    def test_create_subnet_from_pool_with_default_prefixlen(self):
        # If neither cidr nor prefixlen is specified,
        # subnet will use subnetpool default_prefixlen for cidr.
        pool_id, subnet = self._create_subnet_from_pool()
        cidr = str(subnet['cidr'])
        self.assertEqual(pool_id, subnet['subnetpool_id'])
        prefixlen = self._subnetpool_data['min_prefixlen']
        self.assertTrue(cidr.endswith(str(prefixlen)))

    @decorators.idempotent_id('a64af292-ec52-4bde-b654-a6984acaf477')
    def test_create_subnet_from_pool_with_quota(self):
        pool_values = {'default_quota': 4}
        subnet_values = {"prefixlen": self.max_prefixlen}
        pool_id, subnet = self._create_subnet_from_pool(
            subnet_values=subnet_values, pool_values=pool_values)
        cidr = str(subnet['cidr'])
        self.assertEqual(pool_id, subnet['subnetpool_id'])
        self.assertTrue(cidr.endswith(str(self.max_prefixlen)))

    @decorators.idempotent_id('49b44c64-1619-4b29-b527-ffc3c3115dc4')
    @utils.requires_ext(extension='address-scope', service='network')
    def test_create_subnetpool_associate_address_scope(self):
        address_scope = self.create_address_scope(
            name=data_utils.rand_name('smoke-address-scope'),
            ip_version=self._ip_version)
        created_subnetpool = self._create_subnetpool(
            address_scope_id=address_scope['id'])
        body = self.client.show_subnetpool(created_subnetpool['id'])
        self.assertEqual(address_scope['id'],
                         body['subnetpool']['address_scope_id'])

    @decorators.idempotent_id('910b6393-db24-4f6f-87dc-b36892ad6c8c')
    @utils.requires_ext(extension='address-scope', service='network')
    def test_update_subnetpool_associate_address_scope(self):
        address_scope = self.create_address_scope(
            name=data_utils.rand_name('smoke-address-scope'),
            ip_version=self._ip_version)
        created_subnetpool = self._create_subnetpool()
        pool_id = created_subnetpool['id']
        body = self.client.show_subnetpool(pool_id)
        self.assertIsNone(body['subnetpool']['address_scope_id'])
        self.client.update_subnetpool(pool_id,
                                      address_scope_id=address_scope['id'])
        body = self.client.show_subnetpool(pool_id)
        self.assertEqual(address_scope['id'],
                         body['subnetpool']['address_scope_id'])

    @decorators.idempotent_id('18302e80-46a3-4563-82ac-ccd1dd57f652')
    @utils.requires_ext(extension='address-scope', service='network')
    def test_update_subnetpool_associate_another_address_scope(self):
        address_scope = self.create_address_scope(
            name=data_utils.rand_name('smoke-address-scope'),
            ip_version=self._ip_version)
        another_address_scope = self.create_address_scope(
            name=data_utils.rand_name('smoke-address-scope'),
            ip_version=self._ip_version)
        created_subnetpool = self._create_subnetpool(
            address_scope_id=address_scope['id'])
        pool_id = created_subnetpool['id']
        body = self.client.show_subnetpool(pool_id)
        self.assertEqual(address_scope['id'],
                         body['subnetpool']['address_scope_id'])
        self.client.update_subnetpool(
            pool_id, address_scope_id=another_address_scope['id'])
        body = self.client.show_subnetpool(pool_id)
        self.assertEqual(another_address_scope['id'],
                         body['subnetpool']['address_scope_id'])

    @decorators.idempotent_id('f8970048-e41b-42d6-934b-a1297b07706a')
    @utils.requires_ext(extension='address-scope', service='network')
    def test_update_subnetpool_disassociate_address_scope(self):
        address_scope = self.create_address_scope(
            name=data_utils.rand_name('smoke-address-scope'),
            ip_version=self._ip_version)
        created_subnetpool = self._create_subnetpool(
            address_scope_id=address_scope['id'])
        pool_id = created_subnetpool['id']
        body = self.client.show_subnetpool(pool_id)
        self.assertEqual(address_scope['id'],
                         body['subnetpool']['address_scope_id'])
        self.client.update_subnetpool(pool_id,
                                      address_scope_id=None)
        body = self.client.show_subnetpool(pool_id)
        self.assertIsNone(body['subnetpool']['address_scope_id'])

    @decorators.idempotent_id('4c6963c2-f54c-4347-b288-75d18421c4c4')
    @utils.requires_ext(extension='default-subnetpools', service='network')
    def test_tenant_create_non_default_subnetpool(self):
        """Test creates a subnetpool, the "is_default" attribute is False."""
        created_subnetpool = self._create_subnetpool()
        self.assertFalse(created_subnetpool['is_default'])


class SubnetPoolsTestV6(SubnetPoolsTest):

    min_prefixlen = '48'
    max_prefixlen = '64'
    _ip_version = 6
    subnet_cidr = '2001:db8:3::/64'
    new_prefix = '2001:db8:5::/64'
    larger_prefix = '2001:db8::/32'

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        min_prefixlen = '64'
        prefixes = ['2001:db8:3::/48']
        cls._subnetpool_data = {'min_prefixlen': min_prefixlen,
                                'prefixes': prefixes}

    @decorators.idempotent_id('f62d73dc-cf6f-4879-b94b-dab53982bf3b')
    def test_create_dual_stack_subnets_from_subnetpools(self):
        pool_id_v6, subnet_v6 = self._create_subnet_from_pool()
        pool_values_v4 = {'prefixes': ['192.168.0.0/16'],
                          'min_prefixlen': 21,
                          'max_prefixlen': 32}
        create_v4_subnetpool = self._create_subnetpool(**pool_values_v4)
        pool_id_v4 = create_v4_subnetpool['id']
        subnet_v4 = self.client.create_subnet(
            network_id=subnet_v6['network_id'], ip_version=4,
            subnetpool_id=pool_id_v4)['subnet']
        self.assertEqual(subnet_v4['network_id'], subnet_v6['network_id'])


class SubnetPoolsSearchCriteriaTest(base.BaseSearchCriteriaTest,
                                    SubnetPoolsTestBase):

    resource = 'subnetpool'

    list_kwargs = {'shared': False}

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        for name in cls.resource_names:
            cls._create_subnetpool(name=name)

    @decorators.idempotent_id('6e3f842e-6bfb-49cb-82d3-0026be4e8e04')
    def test_list_sorts_asc(self):
        self._test_list_sorts_asc()

    @decorators.idempotent_id('f336859b-b868-438c-a6fc-2c06374115f2')
    def test_list_sorts_desc(self):
        self._test_list_sorts_desc()

    @decorators.idempotent_id('1291fae7-c196-4372-ad59-ce7988518f7b')
    def test_list_pagination(self):
        self._test_list_pagination()

    @decorators.idempotent_id('ddb20d14-1952-49b4-a17e-231cc2239a52')
    def test_list_pagination_with_marker(self):
        self._test_list_pagination_with_marker()

    @decorators.idempotent_id('b3bd9665-2769-4a43-b50c-31b1add12891')
    def test_list_pagination_with_href_links(self):
        self._test_list_pagination_with_href_links()

    @decorators.idempotent_id('1ec1f325-43b0-406e-96ce-20539e38a61d')
    def test_list_pagination_page_reverse_asc(self):
        self._test_list_pagination_page_reverse_asc()

    @decorators.idempotent_id('f43a293e-4aaa-48f4-aeaf-de63a676357c')
    def test_list_pagination_page_reverse_desc(self):
        self._test_list_pagination_page_reverse_desc()

    @decorators.idempotent_id('73511385-839c-4829-8ac1-b5ad992126c4')
    def test_list_pagination_page_reverse_with_href_links(self):
        self._test_list_pagination_page_reverse_with_href_links()

    @decorators.idempotent_id('82a13efc-c18f-4249-b8ec-cec7cf26fbd6')
    def test_list_no_pagination_limit_0(self):
        self._test_list_no_pagination_limit_0()

    @decorators.idempotent_id('27feb3f8-40f4-4e50-8cd2-7d0096a98682')
    def test_list_validation_filters(self):
        self._test_list_validation_filters(self.list_kwargs)
        self._test_list_validation_filters({
            'unknown_filter': 'value'}, filter_is_valid=False)


class RbacSubnetPoolTest(SubnetPoolsTestBase):

    force_tenant_isolation = True
    credentials = ['primary', 'alt', 'admin']
    required_extensions = ['rbac-subnetpool']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.client2 = cls.os_alt.network_client

    def _make_admin_snp_shared_to_project_id(self, project_id):
        snp = self._create_subnetpool(is_admin=True)
        rbac_policy = self.admin_client.create_rbac_policy(
            object_type='subnetpool',
            object_id=snp['id'],
            action='access_as_shared',
            target_tenant=project_id,
        )['rbac_policy']
        return {'subnetpool': snp, 'rbac_policy': rbac_policy}

    @decorators.idempotent_id('71b35ad0-51cd-40da-985d-89a51c95ec6a')
    def test_policy_target_update(self):
        res = self._make_admin_snp_shared_to_project_id(
            self.client.project_id)
        # change to client2
        update_res = self.admin_client.update_rbac_policy(
            res['rbac_policy']['id'], target_tenant=self.client2.project_id)
        self.assertEqual(self.client2.project_id,
                         update_res['rbac_policy']['target_tenant'])
        # make sure everything else stayed the same
        res['rbac_policy'].pop('target_tenant')
        update_res['rbac_policy'].pop('target_tenant')
        self.assertEqual(res['rbac_policy'], update_res['rbac_policy'])

    @decorators.idempotent_id('451d9d38-65a0-4916-a805-1460d6a938d1')
    def test_subnet_presence_prevents_rbac_policy_deletion(self):
        res = self._make_admin_snp_shared_to_project_id(
            self.client2.project_id)
        network = self.create_network(client=self.client2)
        subnet = self.client2.create_subnet(
            network_id=network['id'],
            ip_version=4,
            subnetpool_id=res['subnetpool']['id'],
            name=data_utils.rand_name("rbac-subnetpool"),
        )["subnet"]
        self.addCleanup(self.client2.delete_network, network['id'])
        self.addCleanup(
            self.admin_client.delete_subnetpool,
            res['subnetpool']['id']
        )
        self.addCleanup(self.client2.delete_subnet, subnet['id'])

        # a port with shared sg should prevent the deletion of an
        # rbac-policy required for it to be shared
        with testtools.ExpectedException(lib_exc.Conflict):
            self.admin_client.delete_rbac_policy(res['rbac_policy']['id'])

    @decorators.idempotent_id('f74a71de-9abf-49c6-8199-4ac7f53e383b')
    @utils.requires_ext(extension='rbac-address-scope', service='network')
    def test_cannot_share_if_no_access_to_address_scope(self):
        # Create Address Scope shared only to client but not to client2
        a_s = self.admin_client.create_address_scope(
            name=data_utils.rand_name("rbac-subnetpool"),
            ip_version=4
        )["address_scope"]
        rbac_policy = self.admin_client.create_rbac_policy(
            object_type='address_scope', object_id=a_s['id'],
            action='access_as_shared',
            target_tenant=self.client.project_id)['rbac_policy']

        # Create subnet pool owned by client with shared AS
        snp = self._create_subnetpool(address_scope_id=a_s["id"])

        with testtools.ExpectedException(lib_exc.BadRequest):
            self.client.create_rbac_policy(
                object_type='subnetpool', object_id=snp['id'],
                action='access_as_shared',
                target_tenant=self.client2.project_id
            )

        # cleanup
        self.client.delete_subnetpool(snp["id"])
        self.admin_client.delete_rbac_policy(rbac_policy['id'])
        self.admin_client.delete_address_scope(a_s['id'])

    @decorators.idempotent_id('9cf8bba5-0163-4083-9397-678bb9b5f5a2')
    def test_regular_client_shares_to_another_regular_client(self):
        # owned by self.admin_client
        snp = self._create_subnetpool(is_admin=True)
        with testtools.ExpectedException(lib_exc.NotFound):
            self.client.show_subnetpool(snp['id'])
        rbac_policy = self.admin_client.create_rbac_policy(
            object_type='subnetpool', object_id=snp['id'],
            action='access_as_shared',
            target_tenant=self.client.project_id)['rbac_policy']
        self.client.show_subnetpool(snp['id'])

        self.assertIn(rbac_policy,
                      self.admin_client.list_rbac_policies()['rbac_policies'])
        # ensure that 'client2' can't see the rbac-policy sharing the
        # as to it because the rbac-policy belongs to 'client'
        self.assertNotIn(rbac_policy['id'], [p['id'] for p in
                         self.client2.list_rbac_policies()['rbac_policies']])

    @decorators.idempotent_id('17b2b437-a5fa-4340-ad98-912a986d0d7c')
    def test_filter_fields(self):
        snp = self._create_subnetpool()
        self.admin_client.create_rbac_policy(
            object_type='subnetpool', object_id=snp['id'],
            action='access_as_shared', target_tenant=self.client2.project_id)
        field_args = (('id',), ('id', 'action'), ('object_type', 'object_id'),
                      ('project_id', 'target_tenant'))
        for fields in field_args:
            res = self.admin_client.list_rbac_policies(fields=fields)
            self.assertEqual(set(fields), set(res['rbac_policies'][0].keys()))

    @decorators.idempotent_id('e59e4502-4e6a-4e49-b446-a5d5642bbd69')
    def test_rbac_policy_show(self):
        res = self._make_admin_snp_shared_to_project_id(
            self.client.project_id)
        p1 = res['rbac_policy']
        p2 = self.admin_client.create_rbac_policy(
            object_type='subnetpool',
            object_id=res['subnetpool']['id'],
            action='access_as_shared',
            target_tenant='*')['rbac_policy']

        self.assertEqual(
            p1, self.admin_client.show_rbac_policy(p1['id'])['rbac_policy'])
        self.assertEqual(
            p2, self.admin_client.show_rbac_policy(p2['id'])['rbac_policy'])

    @decorators.idempotent_id('1c24c28c-eb1e-466e-af29-255cf127653a')
    def test_filter_rbac_policies(self):
        snp = self._create_subnetpool()
        rbac_pol1 = self.admin_client.create_rbac_policy(
            object_type='subnetpool', object_id=snp['id'],
            action='access_as_shared',
            target_tenant=self.client2.project_id)['rbac_policy']
        rbac_pol2 = self.admin_client.create_rbac_policy(
            object_type='subnetpool', object_id=snp['id'],
            action='access_as_shared',
            target_tenant=self.admin_client.project_id)['rbac_policy']
        res1 = self.admin_client.list_rbac_policies(id=rbac_pol1['id'])[
            'rbac_policies']
        res2 = self.admin_client.list_rbac_policies(id=rbac_pol2['id'])[
            'rbac_policies']
        self.assertEqual(1, len(res1))
        self.assertEqual(1, len(res2))
        self.assertEqual(rbac_pol1['id'], res1[0]['id'])
        self.assertEqual(rbac_pol2['id'], res2[0]['id'])

    @decorators.idempotent_id('63d9acbe-403c-4e77-9ffd-80e636a4621e')
    def test_regular_client_blocked_from_sharing_anothers_policy(self):
        snp = self._make_admin_snp_shared_to_project_id(
            self.client.project_id)['subnetpool']
        with testtools.ExpectedException(lib_exc.BadRequest):
            self.client.create_rbac_policy(
                object_type='subnetpool', object_id=snp['id'],
                action='access_as_shared',
                target_tenant=self.client2.project_id)

        # make sure the rbac-policy is invisible to the tenant for which it's
        # being shared
        self.assertFalse(self.client.list_rbac_policies()['rbac_policies'])



# Copyright 2016 IBM
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import auto_allocated_topology
from oslo_config import cfg
from tempest.lib import decorators

from neutron_tempest_plugin.api import base


class TestAutoAllocatedTopology(base.BaseAdminNetworkTest):
    """Tests Get-Me-A-Network

    Tests the Get-Me-A-Network operations in the Neutron API
    using the REST client for Neutron.
    """
    # NOTE(armax): this is a precaution to avoid interference
    # from other tests exercising this extension. So long as
    # all tests are added under TestAutoAllocatedTopology,
    # nothing bad should happen.
    force_tenant_isolation = True
    required_extensions = [auto_allocated_topology.ALIAS]

    @classmethod
    def resource_setup(cls):
        super().resource_setup()

        # The deployment must contain a default subnetpool
        body = cls.client.list_subnetpools(is_default=True)
        # The deployment may contain one or two default subnetpools:
        # one ipv4 pool, or one ipv6 pool, or one of each.
        # This run-time dependency should be revisited if the test is
        # moved over to tempest.
        cls.num_subnetpools = len(body['subnetpools'])
        if cls.num_subnetpools == 0:
            raise cls.skipException("No default subnetpool")

        # Ensure the public external network is the default external network
        public_net_id = cfg.CONF.network.public_network_id
        cls.admin_client.update_network(public_net_id, is_default=True)
        # Ensure that is_default does not accidentally flip back to False
        # because of network_update requests that do not contain is_default.
        cls.admin_client.update_network(public_net_id, description="gman")

    def _count_topology_resources(self):
        '''Count the resources whose names begin with 'auto_allocated_'.'''

        def _count(resources):
            return len([resource['id'] for resource in resources
                        if resource['name'].startswith('auto_allocated_')])

        up = {'admin_state_up': True}
        networks = _count(self.client.list_networks(**up)['networks'])
        subnets = _count(self.client.list_subnets()['subnets'])
        routers = _count(self.client.list_routers(**up)['routers'])
        return networks, subnets, routers

    def _add_topology_cleanup(self, client):
        '''Add the auto-allocated resources to the cleanup lists.'''

        body = client.list_routers(name='auto_allocated_router')
        self.routers.extend(body['routers'])
        body = client.list_subnets(name='auto_allocated_subnet_v4')
        self.subnets.extend(body['subnets'])
        body = client.list_subnets(name='auto_allocated_subnet_v6')
        self.subnets.extend(body['subnets'])
        body = client.list_networks(name='auto_allocated_network')
        self.networks.extend(body['networks'])

    @decorators.idempotent_id('10e4ecef-4309-4bda-9c4d-192e1b5a5831')
    def test_check_requirements_for_auto_allocate_net_topology(self):
        body = self.client.validate_auto_allocated_topology_requirements()
        topology = body['auto_allocated_topology']
        self.assertIn('dry-run', topology['id'])
        self.assertIn('project_id', topology)

    @decorators.idempotent_id('64bc0b02-cee4-11e5-9f3c-080027605a2b')
    def test_get_allocated_net_topology_as_project(self):
        resources_before = self._count_topology_resources()
        self.assertEqual((0, 0, 0), resources_before)

        body = self.client.get_auto_allocated_topology()
        topology = body[auto_allocated_topology.RESOURCE_NAME]
        self.assertIsNotNone(topology)
        self._add_topology_cleanup(self.client)

        network_id1 = topology['id']
        self.assertIsNotNone(network_id1)
        network = self.client.show_network(topology['id'])['network']
        self.assertTrue(network['admin_state_up'])
        resources_after1 = self._count_topology_resources()
        # One network, two subnets (v4 and v6) and one router
        self.assertEqual((1, self.num_subnetpools, 1), resources_after1)

        body = self.client.get_auto_allocated_topology()
        topology = body[auto_allocated_topology.RESOURCE_NAME]
        network_id2 = topology['id']
        resources_after2 = self._count_topology_resources()
        # After the initial GET, the API should be idempotent
        self.assertEqual(network_id1, network_id2)
        self.assertEqual(resources_after1, resources_after2)

    @decorators.idempotent_id('aabc0b02-cee4-11e5-9f3c-091127605a2b')
    def test_delete_allocated_net_topology_as_project(self):
        resources_before = self._count_topology_resources()
        self.assertEqual((0, 0, 0), resources_before)
        body = self.client.get_auto_allocated_topology()
        topology = body[auto_allocated_topology.RESOURCE_NAME]
        self.assertIsNotNone(topology)
        self.client.delete_auto_allocated_topology()
        resources_after = self._count_topology_resources()
        self.assertEqual((0, 0, 0), resources_after)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from neutron_lib import constants as const

from tempest.common import utils as tutils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api import base
from neutron_tempest_plugin.api import base_routers
from neutron_tempest_plugin.common import utils
from neutron_tempest_plugin import config

CONF = config.CONF


class RoutersTest(base_routers.BaseRouterTest):

    required_extensions = ['router']

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.tenant_cidr = (
            CONF.network.project_network_cidr
            if cls._ip_version == 4 else
            CONF.network.project_network_v6_cidr)

    @decorators.idempotent_id('c72c1c0c-2193-4aca-eeee-b1442640eeee')
    @tutils.requires_ext(extension="standard-attr-description",
                       service="network")
    def test_create_update_router_description(self):
        body = self.create_router(description='d1', router_name='test')
        self.assertEqual('d1', body['description'])
        body = self.client.show_router(body['id'])['router']
        self.assertEqual('d1', body['description'])
        body = self.client.update_router(body['id'], description='d2')
        self.assertEqual('d2', body['router']['description'])
        body = self.client.show_router(body['router']['id'])['router']
        self.assertEqual('d2', body['description'])

    @decorators.idempotent_id('847257cc-6afd-4154-b8fb-af49f5670ce8')
    @tutils.requires_ext(extension='ext-gw-mode', service='network')
    def test_create_router_with_default_snat_value(self):
        # Create a router with default snat rule
        name = data_utils.rand_name('router')
        router = self._create_router(
            name, external_network_id=CONF.network.public_network_id)
        self._verify_router_gateway(
            router['id'], {'network_id': CONF.network.public_network_id,
                           'enable_snat': True})

    @decorators.idempotent_id('ea74068d-09e9-4fd7-8995-9b6a1ace920f')
    @tutils.requires_ext(extension='ext-gw-mode', service='network')
    def test_create_router_with_snat_explicit(self):
        name = data_utils.rand_name('snat-router')
        # Create a router enabling snat attributes
        enable_snat_states = [False, True]
        for enable_snat in enable_snat_states:
            external_gateway_info = {
                'network_id': CONF.network.public_network_id,
                'enable_snat': enable_snat}
            router = self._create_admin_router(
                name, external_network_id=CONF.network.public_network_id,
                enable_snat=enable_snat)
            # Verify snat attributes after router creation
            self._verify_router_gateway(router['id'],
                                        exp_ext_gw_info=external_gateway_info)

    def _verify_router_gateway(self, router_id, exp_ext_gw_info=None):
        show_body = self.admin_client.show_router(router_id)
        actual_ext_gw_info = show_body['router']['external_gateway_info']
        if exp_ext_gw_info is None:
            self.assertIsNone(actual_ext_gw_info)
            return
        # Verify only keys passed in exp_ext_gw_info
        for k, v in exp_ext_gw_info.items():
            self.assertEqual(v, actual_ext_gw_info[k])

    def _verify_gateway_port(self, router_id):
        list_body = self.admin_client.list_ports(
            network_id=CONF.network.public_network_id,
            device_id=router_id)
        self.assertEqual(len(list_body['ports']), 1)
        gw_port = list_body['ports'][0]
        fixed_ips = gw_port['fixed_ips']
        self.assertGreaterEqual(len(fixed_ips), 1)
        public_net_body = self.admin_client.show_network(
            CONF.network.public_network_id)
        public_subnet_ids = public_net_body['network']['subnets']
        for fixed_ip in fixed_ips:
            self.assertIn(fixed_ip['subnet_id'],
                          public_subnet_ids)

    @decorators.idempotent_id('b386c111-3b21-466d-880c-5e72b01e1a33')
    @tutils.requires_ext(extension='ext-gw-mode', service='network')
    def test_update_router_set_gateway_with_snat_explicit(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.admin_client.update_router_with_snat_gw_info(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': True})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': True})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('96536bc7-8262-4fb2-9967-5c46940fa279')
    @tutils.requires_ext(extension='ext-gw-mode', service='network')
    def test_update_router_set_gateway_without_snat(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.admin_client.update_router_with_snat_gw_info(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': False})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': False})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('f2faf994-97f4-410b-a831-9bc977b64374')
    @tutils.requires_ext(extension='ext-gw-mode', service='network')
    def test_update_router_reset_gateway_without_snat(self):
        router = self._create_router(
            data_utils.rand_name('router'),
            external_network_id=CONF.network.public_network_id)
        self.admin_client.update_router_with_snat_gw_info(
            router['id'],
            external_gateway_info={
                'network_id': CONF.network.public_network_id,
                'enable_snat': False})
        self._verify_router_gateway(
            router['id'],
            {'network_id': CONF.network.public_network_id,
             'enable_snat': False})
        self._verify_gateway_port(router['id'])

    @decorators.idempotent_id('db3093b1-93b6-4893-be83-c4716c251b3e')
    def test_router_interface_status(self):
        network = self.create_network()
        subnet = self.create_subnet(network)
        # Add router interface with subnet id
        router = self._create_router(data_utils.rand_name('router'), True)
        intf = self.create_router_interface(router['id'], subnet['id'])

        def _status_active():
            return self.client.show_port(
                intf['port_id'])['port']['status'] == 'ACTIVE'

        utils.wait_until_true(_status_active, exception=AssertionError)

    @decorators.idempotent_id('c86ac3a8-50bd-4b00-a6b8-62af84a0765c')
    @tutils.requires_ext(extension='extraroute', service='network')
    def test_update_extra_route(self):
        self.network = self.create_network()
        self.name = self.network['name']
        self.subnet = self.create_subnet(self.network)
        # Add router interface with subnet id
        self.router = self._create_router(
            data_utils.rand_name('router-'), True)
        self.create_router_interface(self.router['id'], self.subnet['id'])
        self.addCleanup(
            self._delete_extra_routes,
            self.router['id'])
        # Update router extra route, second ip of the range is
        # used as next hop
        cidr = netaddr.IPNetwork(self.subnet['cidr'])
        next_hop = str(cidr[2])
        destination = str(self.subnet['cidr'])
        extra_route = self.client.update_extra_routes(self.router['id'],
                                                      next_hop, destination)
        self.assertEqual(1, len(extra_route['router']['routes']))
        self.assertEqual(destination,
                         extra_route['router']['routes'][0]['destination'])
        self.assertEqual(next_hop,
                         extra_route['router']['routes'][0]['nexthop'])
        show_body = self.client.show_router(self.router['id'])
        self.assertEqual(destination,
                         show_body['router']['routes'][0]['destination'])
        self.assertEqual(next_hop,
                         show_body['router']['routes'][0]['nexthop'])

    def _delete_extra_routes(self, router_id):
        self.client.delete_extra_routes(router_id)

    @decorators.idempotent_id('b29d1698-d603-11e9-9c66-079cc4aec539')
    @tutils.requires_ext(extension='extraroute-atomic', service='network')
    def test_extra_routes_atomic(self):
        self.network = self.create_network()
        self.subnet = self.create_subnet(self.network)
        self.router = self._create_router(
            data_utils.rand_name('router-'), True)
        self.create_router_interface(self.router['id'], self.subnet['id'])
        self.addCleanup(
            self._delete_extra_routes,
            self.router['id'])

        if self._ip_version == 6:
            dst = '2001:db8:%s::/64'
        else:
            dst = '10.0.%s.0/24'

        cidr = netaddr.IPNetwork(self.subnet['cidr'])

        routes = [
            {'destination': dst % 2, 'nexthop': cidr[2]},
        ]
        resp = self.client.add_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(1, len(resp['router']['routes']))

        routes = [
            {'destination': dst % 2, 'nexthop': cidr[2]},
            {'destination': dst % 3, 'nexthop': cidr[3]},
        ]
        resp = self.client.add_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(2, len(resp['router']['routes']))

        routes = [
            {'destination': dst % 3, 'nexthop': cidr[3]},
            {'destination': dst % 4, 'nexthop': cidr[4]},
        ]
        resp = self.client.remove_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(1, len(resp['router']['routes']))

        routes = [
            {'destination': dst % 2, 'nexthop': cidr[5]},
        ]
        resp = self.client.add_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(2, len(resp['router']['routes']))

        routes = [
            {'destination': dst % 2, 'nexthop': cidr[5]},
        ]
        resp = self.client.remove_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(1, len(resp['router']['routes']))

        routes = [
            {'destination': dst % 2, 'nexthop': cidr[2]},
            {'destination': dst % 3, 'nexthop': cidr[3]},
            {'destination': dst % 2, 'nexthop': cidr[5]},
        ]
        resp = self.client.remove_extra_routes_atomic(
            self.router['id'], routes)
        self.assertEqual(0, len(resp['router']['routes']))

    @decorators.idempotent_id('01f185d1-d1a6-4cf9-abf7-e0e1384c169c')
    def test_network_attached_with_two_routers(self):
        network = self.create_network(data_utils.rand_name('network1'))
        self.create_subnet(network)
        port1 = self.create_port(network)
        port2 = self.create_port(network)
        router1 = self._create_router(data_utils.rand_name('router1'))
        router2 = self._create_router(data_utils.rand_name('router2'))
        self.client.add_router_interface_with_port_id(
            router1['id'], port1['id'])
        self.client.add_router_interface_with_port_id(
            router2['id'], port2['id'])
        self.addCleanup(self.client.remove_router_interface_with_port_id,
                        router1['id'], port1['id'])
        self.addCleanup(self.client.remove_router_interface_with_port_id,
                        router2['id'], port2['id'])
        body = self.client.show_port(port1['id'])
        port_show1 = body['port']
        body = self.client.show_port(port2['id'])
        port_show2 = body['port']
        self.assertEqual(port_show1['network_id'], network['id'])
        self.assertEqual(port_show2['network_id'], network['id'])
        self.assertEqual(port_show1['device_id'], router1['id'])
        self.assertEqual(port_show2['device_id'], router2['id'])

    @decorators.idempotent_id('4f8a2a1e-7fe9-4d99-9bff-5dc0e78b7e06')
    def test_router_interface_update_and_remove_gateway_ip(self):
        network = self.create_network()
        subnet = self.create_subnet(network, allocation_pool_size=5)

        # Update the subnet gateway IP, using the next one. Because the
        # allocation pool is on the upper part of the CIDR, the lower IP
        # addresses are free. This operation must be allowed because the subnet
        # does not have yet a router port.
        gateway_ip = netaddr.IPAddress(subnet['gateway_ip'])
        self.client.update_subnet(subnet['id'], gateway_ip=str(gateway_ip + 1))

        router = self._create_router(data_utils.rand_name('router'), True)
        intf = self.create_router_interface(router['id'], subnet['id'])

        def _status_active():
            return self.client.show_port(
                intf['port_id'])['port']['status'] == 'ACTIVE'

        utils.wait_until_true(_status_active, exception=AssertionError)

        # The gateway update must raise a ``GatewayIpInUse`` exception because
        # there is an allocated router port.
        gateway_ip = netaddr.IPAddress(subnet['gateway_ip'])
        self.assertRaises(lib_exc.Conflict, self.client.update_subnet,
                          subnet['id'], gateway_ip=str(gateway_ip + 2))

        # The gateway deletion returns the same exception.
        gateway_ip = netaddr.IPAddress(subnet['gateway_ip'])
        self.assertRaises(lib_exc.Conflict, self.client.update_subnet,
                          subnet['id'], gateway_ip=None)


class ExternalGWMultihomingRoutersTest(base_routers.BaseRouterTest):

    @classmethod
    @tutils.requires_ext(extension="external-gateway-multihoming",
                         service="network")
    def setUpClass(cls):
        super().setUpClass()

    @decorators.idempotent_id('33e9a156-a83f-435f-90ee-1a49dc9c350d')
    def test_create_router_enable_default_route_ecmp(self):
        router1 = self._create_admin_router(data_utils.rand_name('router1'),
                                            enable_default_route_ecmp=True)
        router2 = self._create_admin_router(data_utils.rand_name('router2'),
                                            enable_default_route_ecmp=False)
        self.assertEqual(router1['enable_default_route_ecmp'], True)
        self.assertEqual(router2['enable_default_route_ecmp'], False)

    @decorators.idempotent_id('bfbad985-2df2-4cd9-9c32-819b5508c40e')
    def test_update_router_enable_default_route_ecmp(self):
        router = self._create_router(data_utils.rand_name('router'))
        updated_router = self.admin_client.update_router(
            router['id'],
            enable_default_route_ecmp=not router['enable_default_route_ecmp'])
        self.assertNotEqual(
            router['enable_default_route_ecmp'],
            updated_router['router']['enable_default_route_ecmp'])

    @decorators.idempotent_id('a22016a6-f118-4eb5-abab-7e241ae01848')
    def test_update_router_enable_default_route_bfd(self):
        router = self._create_router(data_utils.rand_name('router'))
        updated_router = self.admin_client.update_router(
            router['id'],
            enable_default_route_bfd=not router['enable_default_route_bfd'])
        self.assertNotEqual(
            router['enable_default_route_bfd'],
            updated_router['router']['enable_default_route_bfd'])

    @decorators.idempotent_id('842f6edb-e072-4805-bf11-04c25420776d')
    def test_create_router_enable_default_route_bfd(self):
        router1 = self._create_admin_router(data_utils.rand_name('router1'),
                                            enable_default_route_bfd=True)
        router2 = self._create_admin_router(data_utils.rand_name('router2'),
                                            enable_default_route_bfd=False)
        self.assertEqual(router1['enable_default_route_bfd'], True)
        self.assertEqual(router2['enable_default_route_bfd'], False)

    @decorators.idempotent_id('089fa304-3726-4120-9759-668e8ff1114c')
    def test_create_router_add_external_gateways_one(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.assertEqual(len(router['external_gateways']), 0)

        res = self.admin_client.router_add_external_gateways(
            router['id'],
            [{'network_id': CONF.network.public_network_id,
              'enable_snat': False}])
        self.assertEqual(len(res['router']['external_gateways']), 1)
        self.assertEqual(
            res['router']['external_gateways'][0]['network_id'],
            CONF.network.public_network_id)

    @decorators.idempotent_id('60a1e7db-04ef-4a3a-9ff1-01a990d365fd')
    def test_create_router_add_external_gateways(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.assertEqual(len(router['external_gateways']), 0)

        res = self.admin_client.router_add_external_gateways(
            router['id'],
            [
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
            ])
        self.assertEqual(len(res['router']['external_gateways']), 3)
        self.assertEqual(
            res['router']['external_gateway_info']['network_id'],
            res['router']['external_gateways'][0]['network_id'])
        self.assertEqual(
            res['router']['external_gateway_info']['external_fixed_ips'],
            res['router']['external_gateways'][0]['external_fixed_ips'])
        for n in range(0, 3):
            self.assertEqual(
                res['router']['external_gateways'][n]['network_id'],
                CONF.network.public_network_id)
            if n:
                self.assertNotEqual(
                    res['router']['external_gateways'][
                        n]['external_fixed_ips'],
                    res['router']['external_gateways'][
                        n - 1]['external_fixed_ips'])

    @decorators.idempotent_id('e49efc57-7b25-43a3-8e55-2d87a3759c57')
    def test_create_router_add_external_gateways_compat(self):
        router = self._create_router(
            data_utils.rand_name('router'),
            external_network_id=CONF.network.public_network_id,
            enable_snat=False,
            client=self.admin_client,
        )
        self.assertEqual(len(router['external_gateways']), 1)
        res = self.admin_client.router_add_external_gateways(
            router['id'],
            [{'network_id': CONF.network.public_network_id,
              'enable_snat': False}])
        self.assertEqual(len(res['router']['external_gateways']), 2)

    @decorators.idempotent_id('2a238eec-d9d5-435a-9013-d6e195ecd5d1')
    def test_create_router_remove_external_gateways_compat(self):
        router = self._create_router(
            data_utils.rand_name('router'),
            external_network_id=CONF.network.public_network_id,
            enable_snat=False,
            client=self.admin_client)
        self.assertEqual(len(router['external_gateways']), 1)
        res = self.admin_client.router_remove_external_gateways(
            router['id'],
            [{'network_id': CONF.network.public_network_id}])
        self.assertEqual(len(res['router']['external_gateways']), 0)

    @decorators.idempotent_id('03ab196a-dac0-4363-93e4-ea799246870b')
    def test_create_router_add_remove_external_gateways(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.assertEqual(len(router['external_gateways']), 0)

        res = self.admin_client.router_add_external_gateways(
            router['id'],
            [
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
            ])
        self.assertEqual(len(res['router']['external_gateways']), 3)
        remove_gateways = [res['router']['external_gateways'][2]]
        res = self.client.router_remove_external_gateways(router['id'],
                                                          remove_gateways)
        self.assertEqual(len(res['router']['external_gateways']), 2)
        for n in range(0, 2):
            self.assertNotEqual(
                    res['router']['external_gateways'][
                        n]['external_fixed_ips'],
                    remove_gateways[0])

    @decorators.idempotent_id('17e94c9f-c59f-4e50-abd5-d1256460e311')
    def test_create_router_update_external_gateways(self):
        """Add three GW ports, delete last one, re-use IPs in update on second.

        NOTE(fnordahl): Main reason for IP re-use is to ensure we don't tread
        on allocations done by other tests.
        """
        router = self._create_router(data_utils.rand_name('router'))
        self.assertEqual(len(router['external_gateways']), 0)

        res = self.admin_client.router_add_external_gateways(
            router['id'],
            [
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
                {'network_id': CONF.network.public_network_id,
                 'enable_snat': False},
            ])
        self.assertEqual(len(res['router']['external_gateways']), 3)
        external_gateways = res['router']['external_gateways']
        remove_gateways = [external_gateways.pop(2)]
        res_remove_gws = self.client.router_remove_external_gateways(
            router['id'],
            remove_gateways)
        for n in range(0, 2):
            self.assertNotEqual(
                    res_remove_gws['router']['external_gateways'][
                        n]['external_fixed_ips'],
                    remove_gateways[0])

        external_gateways[1] = remove_gateways[0]
        try:
            res_update_gws = self.admin_client.router_update_external_gateways(
                router['id'],
                external_gateways)
        except lib_exc.Conflict as exc:
            if 'IpAddressAlreadyAllocated' in str(exc):
                self.skipTest(
                    'The IP address of the removed gateway port is already '
                    'used by other test, thus this exception is dismissed and '
                    'the rest of the test skipped')
            raise exc

        self.assertEqual(len(res_update_gws['router']['external_gateways']), 2)
        for n in range(0, 2):
            if res_update_gws['router']['external_gateways'][
                        n] == remove_gateways[0]:
                break
        else:
            self.fail('%s not in %s' % (
                remove_gateways[0],
                res_update_gws['router']['external_gateways']))


class RoutersIpV6Test(RoutersTest):
    _ip_version = 6


class DvrRoutersTest(base_routers.BaseRouterTest):

    required_extensions = ['dvr']

    @decorators.idempotent_id('141297aa-3424-455d-aa8d-f2d95731e00a')
    def test_create_distributed_router(self):
        name = data_utils.rand_name('router')
        router = self._create_admin_router(name, distributed=True)
        self.assertTrue(router['distributed'])


class DvrRoutersTestToCentralized(base_routers.BaseRouterTest):

    required_extensions = ['dvr', 'l3-ha']

    @decorators.idempotent_id('644d7a4a-01a1-4b68-bb8d-0c0042cb1729')
    def test_convert_distributed_router_back_to_centralized(self):
        # Convert a centralized router to distributed firstly
        router_args = {'tenant_id': self.client.project_id,
                       'distributed': False, 'ha': False}
        router = self._create_admin_router(
            data_utils.rand_name('router'), admin_state_up=False,
            **router_args)
        self.assertFalse(router['distributed'])
        self.assertFalse(router['ha'])
        update_body = self.admin_client.update_router(router['id'],
                                                      distributed=True)
        self.assertTrue(update_body['router']['distributed'])
        show_body = self.admin_client.show_router(router['id'])
        self.assertTrue(show_body['router']['distributed'])
        self.assertFalse(show_body['router']['ha'])
        # Then convert the distributed router back to centralized
        update_body = self.admin_client.update_router(router['id'],
                                                      distributed=False)
        self.assertFalse(update_body['router']['distributed'])
        show_body = self.admin_client.show_router(router['id'])
        self.assertFalse(show_body['router']['distributed'])
        self.assertFalse(show_body['router']['ha'])
        show_body = self.client.show_router(router['id'])
        self.assertNotIn('distributed', show_body['router'])
        self.assertNotIn('ha', show_body['router'])


class DvrRoutersTestUpdateDistributedExtended(base_routers.BaseRouterTest):

    required_extensions = ['dvr', 'l3-ha',
                           'router-admin-state-down-before-update']

    @decorators.idempotent_id('0ffb9973-0c1a-4b76-a1f2-060178057661')
    def test_convert_centralized_router_to_distributed_extended(self):
        router_args = {'tenant_id': self.client.project_id,
                       'distributed': False, 'ha': False}
        router = self._create_admin_router(
            data_utils.rand_name('router'), admin_state_up=True,
            **router_args)
        self.assertTrue(router['admin_state_up'])
        self.assertFalse(router['distributed'])
        # take router down to allow setting the router to distributed
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=False)
        self.assertFalse(update_body['router']['admin_state_up'])
        # set the router to distributed
        update_body = self.admin_client.update_router(router['id'],
                                                      distributed=True)
        self.assertTrue(update_body['router']['distributed'])
        # bring the router back up
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=True)
        self.assertTrue(update_body['router']['admin_state_up'])
        self.assertTrue(update_body['router']['distributed'])

    @decorators.idempotent_id('e9a8f55b-c535-44b7-8b0a-20af6a7c2921')
    def test_convert_distributed_router_to_centralized_extended(self):
        router_args = {'tenant_id': self.client.project_id,
                       'distributed': True, 'ha': False}
        router = self._create_admin_router(
            data_utils.rand_name('router'), admin_state_up=True,
            **router_args)
        self.assertTrue(router['admin_state_up'])
        self.assertTrue(router['distributed'])
        # take router down to allow setting the router to centralized
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=False)
        self.assertFalse(update_body['router']['admin_state_up'])
        # set router to centralized
        update_body = self.admin_client.update_router(router['id'],
                                                      distributed=False)
        self.assertFalse(update_body['router']['distributed'])
        # bring router back up
        update_body = self.admin_client.update_router(router['id'],
                                                      admin_state_up=True)
        self.assertTrue(update_body['router']['admin_state_up'])
        self.assertFalse(update_body['router']['distributed'])


class HaRoutersTest(base_routers.BaseRouterTest):

    required_extensions = ['l3-ha']

    @decorators.idempotent_id('77db8eae-3aa3-4e61-bf2a-e739ce042e53')
    def test_convert_legacy_router(self):
        router = self._create_router(data_utils.rand_name('router'))
        self.assertNotIn('ha', router)
        update_body = self.admin_client.update_router(router['id'],
                                                      ha=True)
        self.assertTrue(update_body['router']['ha'])
        show_body = self.admin_client.show_router(router['id'])
        self.assertTrue(show_body['router']['ha'])
        show_body = self.client.show_router(router['id'])
        self.assertNotIn('ha', show_body['router'])


class RoutersSearchCriteriaTest(base.BaseSearchCriteriaTest):

    required_extensions = ['router']
    resource = 'router'

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        for name in cls.resource_names:
            cls.create_router(router_name=name)

    @decorators.idempotent_id('03a69efb-90a7-435b-bb5c-3add3612085a')
    def test_list_sorts_asc(self):
        self._test_list_sorts_asc()

    @decorators.idempotent_id('95913d30-ff41-4b17-9f44-5258c651e78c')
    def test_list_sorts_desc(self):
        self._test_list_sorts_desc()

    @decorators.idempotent_id('7f7d40b1-e165-4817-8dc5-02f8e2f0dff3')
    def test_list_pagination(self):
        self._test_list_pagination()

    @decorators.idempotent_id('a5b83e83-3d98-45bb-a2c7-0ee179ffd42c')
    def test_list_pagination_with_marker(self):
        self._test_list_pagination_with_marker()

    @decorators.idempotent_id('40804af8-c25d-45f8-b8a8-b4c70345215d')
    def test_list_pagination_with_href_links(self):
        self._test_list_pagination_with_href_links()

    @decorators.idempotent_id('77b9676c-d3cb-43af-a0e8-a5b8c6099e70')
    def test_list_pagination_page_reverse_asc(self):
        self._test_list_pagination_page_reverse_asc()

    @decorators.idempotent_id('3133a2c5-1bb9-4fc7-833e-cf9a1d160255')
    def test_list_pagination_page_reverse_desc(self):
        self._test_list_pagination_page_reverse_desc()

    @decorators.idempotent_id('8252e2f0-b3da-4738-8e25-f6f8d878a2da')
    def test_list_pagination_page_reverse_with_href_links(self):
        self._test_list_pagination_page_reverse_with_href_links()

    @decorators.idempotent_id('fb102124-20f8-4cb3-8c81-f16f5e41d192')
    def test_list_no_pagination_limit_0(self):
        self._test_list_no_pagination_limit_0()


class RoutersDeleteTest(base_routers.BaseRouterTest):
    """The only test in this class is a test that removes router!

    * We cannot delete common and mandatory resources (router in this case)
    * using the existing classes, as it will cause failure in other tests
    * running in parallel.
    """
    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.secgroup = cls.create_security_group(
            name=data_utils.rand_name("test_port_secgroup"))
        router_kwargs = {
            'router_name': data_utils.rand_name('router_to_delete'),
            'external_network_id': CONF.network.public_network_id}
        cls.router = cls.create_router(**router_kwargs)

    @decorators.idempotent_id('dbbc5c74-63c8-11eb-8881-74e5f9e2a801')
    def test_delete_router(self):
        # Create a port on tenant network and associate to the router.
        # Try to delete router. Expected result: "Conflict Error" is raised.
        network = self.create_network()
        subnet = self.create_subnet(network)
        self.create_router_interface(self.router['id'], subnet['id'])
        port = self.create_port(
            network, name=data_utils.rand_name("port"),
            security_groups=[self.secgroup['id']])
        self.create_floatingip(port=port)
        self.assertRaises(
            lib_exc.Conflict, self.client.delete_router, self.router['id'])
        # Delete the associated port
        # Try to delete router. Expected result: "Conflict Error" is raised.
        # Note: there are still interfaces in use.
        self.client.delete_port(port['id'])
        self.assertRaises(
            lib_exc.Conflict, self.client.delete_router, self.router['id'])
        # Delete the rest of the router's ports
        # Try to delete router. Expected result: "PASS"
        interfaces = [
            port for port in self.client.list_router_interfaces(
                self.router['id'])['ports']
            if port['device_owner'] in const.ROUTER_INTERFACE_OWNERS]
        for i in interfaces:
            try:
                self.assertRaises(
                    lib_exc.Conflict, self.client.delete_router,
                    self.router['id'])
                self.client.remove_router_interface_with_subnet_id(
                    self.router['id'], i['fixed_ips'][0]['subnet_id'])
            except lib_exc.NotFound:
                pass
        self.client.delete_router(self.router['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from neutron_tempest_plugin.api.admin import test_quotas
from neutron_tempest_plugin import config

CONF = config.CONF


class QuotasAdminNegativeTestJSON(test_quotas.QuotasTestBase):

    @decorators.attr(type='negative')
    @decorators.idempotent_id('952f9b24-9156-4bdc-90f3-682a3d4302f0')
    def test_create_network_when_quotas_is_full(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'network': 1}
        self._setup_quotas(tenant_id, **new_quotas)

        net_args = {'tenant_id': tenant_id}
        net = self.admin_client.create_network(**net_args)['network']
        self.addCleanup(self.admin_client.delete_network, net['id'])

        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.create_network, **net_args)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('9f676a6e-d729-428b-adcd-4de2867c50e6')
    def test_set_network_quota_lower_than_networks_amount(self):
        tenant_id = self.create_project()['id']
        high_quota = 3
        low_quota = 1
        new_quotas = {'network': high_quota}
        self._setup_quotas(tenant_id, **new_quotas)
        for _ in range(high_quota - 1):
            self._create_network(tenant_id)
        # TODO(mblue): remove check_limit=True when it is default
        new_quotas.update({'network': low_quota, 'check_limit': True})

        self.assertRaises(lib_exc.BadRequest,
                          self.admin_client.update_quotas,
                          tenant_id, **new_quotas)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('0b7f99e3-9f77-45ce-9a89-b39a184de618')
    def test_create_subnet_when_quotas_is_full(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'subnet': 1}
        self._setup_quotas(tenant_id, **new_quotas)

        net_args = {'tenant_id': tenant_id}
        net = self.admin_client.create_network(**net_args)['network']
        self.addCleanup(self.admin_client.delete_network, net['id'])

        subnet_args = {'tenant_id': tenant_id,
                       'network_id': net['id'],
                       'cidr': '10.0.0.0/24',
                       'ip_version': '4'}
        subnet = self.admin_client.create_subnet(**subnet_args)['subnet']
        self.addCleanup(self.admin_client.delete_subnet, subnet['id'])

        subnet_args['cidr'] = '10.1.0.0/24'
        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.create_subnet, **subnet_args)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('fe20d9f9-346c-4a20-bbfa-d9ca390f4dc6')
    def test_create_port_when_quotas_is_full(self):
        tenant_id = self.create_project()['id']
        net_args = {'tenant_id': tenant_id}
        net = self.admin_client.create_network(**net_args)['network']
        self.addCleanup(self.admin_client.delete_network, net['id'])

        subnet_args = {'tenant_id': tenant_id,
                       'network_id': net['id'],
                       'enable_dhcp': False,
                       'cidr': '10.0.0.0/24',
                       'ip_version': '4'}
        subnet = self.admin_client.create_subnet(**subnet_args)['subnet']
        self.addCleanup(self.admin_client.delete_subnet, subnet['id'])

        ports = self.admin_client.list_ports(tenant_id=tenant_id)
        quota_limit = len(ports['ports']) + 1
        new_quotas = {'port': quota_limit}
        self._setup_quotas(tenant_id, **new_quotas)

        port_args = {'tenant_id': tenant_id,
                     'network_id': net['id']}
        port = self.admin_client.create_port(**port_args)['port']
        self.addCleanup(self.admin_client.delete_port, port['id'])

        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.create_port, **port_args)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('bb1e9c3c-7e6f-41f1-b579-63dbc655ecb7')
    @utils.requires_ext(extension="router", service="network")
    def test_create_router_when_quotas_is_full(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'router': 1}
        self._setup_quotas(tenant_id, **new_quotas)

        name = data_utils.rand_name('test_router')
        router_args = {'tenant_id': tenant_id}
        router = self.admin_client.create_router(
            name, True, **router_args)['router']
        self.addCleanup(self.admin_client.delete_router, router['id'])

        self.assertRaises(lib_exc.Conflict,
                          self.admin_client.create_router,
                          name, True, **router_args)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('5c924ff7-b7a9-474f-92a3-dbe0f976ec13')
    @utils.requires_ext(extension="security-group", service="network")
    def test_create_security_group_when_quotas_is_full(self):
        project = self.create_project()

        # Set quotas to allow to create only one more security group
        security_groups = self.admin_client.list_security_groups(
            tenant_id=project['id'])['security_groups']
        self._setup_quotas(project['id'],
                           security_group=len(security_groups) + 1)

        self.create_security_group(project=project)
        self.assertRaises(lib_exc.Conflict, self.create_security_group,
                          project=project)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('b7143480-6118-4ed4-be38-1b6f15f30d05')
    @utils.requires_ext(extension="security-group", service="network")
    def test_create_security_group_rule_when_quotas_is_full(self):
        project = self.create_project()
        security_group = self.create_security_group(project=project)

        # Set quotas to allow to create only one more security group rule
        security_group_rules = self.admin_client.list_security_group_rules(
            tenant_id=project['id'])['security_group_rules']
        self._setup_quotas(project['id'],
                           security_group_rule=len(security_group_rules) + 1)

        self.create_security_group_rule(
            project=project, security_group=security_group,
            direction=constants.INGRESS_DIRECTION)
        self.assertRaises(lib_exc.Conflict,
                          self.create_security_group_rule,
                          project=project, security_group=security_group,
                          direction=constants.EGRESS_DIRECTION)

    @decorators.attr(type='negative')
    @decorators.idempotent_id('d00fe5bb-9db8-4e1a-9c31-490f52897e6f')
    @utils.requires_ext(extension="router", service="network")
    def test_create_floatingip_when_quotas_is_full(self):
        tenant_id = self.create_project()['id']
        new_quotas = {'floatingip': 1}
        self._setup_quotas(tenant_id, **new_quotas)

        self.create_floatingip(client=self.admin_client, tenant_id=tenant_id)

        self.assertRaises(lib_exc.Conflict, self.create_floatingip,
                          client=self.admin_client, tenant_id=tenant_id)
